@@ -1,21 +1,25 @@
 class RemoteSequence(object):
-    def __init__(self, vim, remote_klass, length_fn):
-        self._vim = vim
-        self._remote_klass = remote_klass
-        self._length_fn = length_fn
+    # TODO Need to add better support for this class on the server
+    def __init__(self, vim, remote_klass, handle_array_fn):
+        self._wrap_fn = lambda handle: remote_klass(vim, handle)
+        self._handle_array_fn = handle_array_fn
 
     def __len__(self):
-        return self._length_fn()
+        return len(self._handle_array_fn())
+
+    def __getitem__(self, idx):
+        if not isinstance(idx, slice):
+            return self._wrap_fn(self._handle_array_fn()[idx])
+        return map(self._wrap_fn, self._handle_array_fn()[idx.start:idx.stop])
 
-    def __getitem__(self, key):
-        return self._remote_klass(self._vim, key + 1)
 
     def __iter__(self):
-        for i in xrange(1, len(self) + 1):
-            yield self.remote_klass(self._vim, i)
+        handles = self._handle_array_fn()
+        for handle in handles:
+            yield self._wrap_fn(handle)
     
     def __contains__(self, item):
-        return item.handle >= 1 and item.handle <= len(self) 
+        return item._handle in self._handle_array_fn()
 
 
 class RemoteMap(object):
@@ -45,6 +49,9 @@ class RemoteMap(object):
 
 
 class Current(object):
+    def __init__(self, vim):
+        self._vim = vim
+
     @property
     def line(self):
         return self._vim.get_current_line()
